"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _core = require("@babel/core");

var _find = _interopRequireDefault(require("@dword-design/functions/dist/find"));

var _keys = _interopRequireDefault(require("@dword-design/functions/dist/keys"));

var _replace = _interopRequireDefault(require("@dword-design/functions/dist/replace"));

var _some = _interopRequireDefault(require("@dword-design/functions/dist/some"));

var _startsWith = _interopRequireDefault(require("@dword-design/functions/dist/starts-with"));

var _babelPluginModuleResolver = require("babel-plugin-module-resolver");

var _path = _interopRequireDefault(require("path"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const isParentImport = path => /^(\.\/)?\.\.\//.test(path);

const findMatchingAlias = (sourcePath, currentFile, options) => {
  const resolvePath = options.resolvePath || _babelPluginModuleResolver.resolvePath;

  const absoluteSourcePath = _path.default.resolve(_path.default.dirname(currentFile), sourcePath);

  for (const aliasName of (_options$alias = options.alias, (0, _keys.default)(_options$alias))) {
    var _options$alias, _absoluteSourcePath;

    const path = _path.default.resolve(_path.default.dirname(currentFile), resolvePath(`${aliasName}/`, currentFile, options));

    if (_absoluteSourcePath = absoluteSourcePath, (0, _startsWith.default)(path)(_absoluteSourcePath)) {
      return {
        name: aliasName,
        path
      };
    }
  }

  return undefined;
};

var _default = {
  create: context => {
    var _babelConfig$plugins;

    const currentFile = context.getFilename();

    const folder = _path.default.dirname(currentFile); // can't check a non-file


    if (currentFile === '<text>') return {};
    const manager = new _core.OptionManager();
    const babelConfig = manager.init({
      babelrc: true,
      filename: currentFile,
      root: folder
    });
    const plugin = (_babelConfig$plugins = babelConfig.plugins, (0, _find.default)({
      key: 'module-resolver'
    })(_babelConfig$plugins));
    const options = { ...(plugin === null || plugin === void 0 ? void 0 : plugin.options),
      ...context.options[0]
    };
    const resolvePath = options.resolvePath || _babelPluginModuleResolver.resolvePath;
    return {
      ImportDeclaration: node => {
        var _ref, _options$alias2, _sourcePath2, _importWithoutAlias;

        const sourcePath = node.source.value;
        const hasAlias = (_ref = (_options$alias2 = options.alias, (0, _keys.default)(_options$alias2)), (0, _some.default)(alias => {
          var _sourcePath;

          return _sourcePath = sourcePath, (0, _startsWith.default)(`${alias}/`)(_sourcePath);
        })(_ref)); // relative parent

        if (_sourcePath2 = sourcePath, isParentImport(_sourcePath2)) {
          var _P$relative;

          const matchingAlias = findMatchingAlias(sourcePath, currentFile, options);

          if (!matchingAlias) {
            return context.report({
              message: `Unexpected parent import '${sourcePath}'. No matching alias found to fix the issue. You have to define aliases by either passing them to the babel-plugin-module-resolver plugin in your Babel config, or directly to the prefer-alias rule.`,
              node
            });
          }

          const absoluteImportPath = _path.default.resolve(folder, sourcePath);

          const rewrittenImport = `${matchingAlias.name}/${(_P$relative = _path.default.relative(matchingAlias.path, absoluteImportPath), (0, _replace.default)(/\\/g, '/')(_P$relative))}`;
          return context.report({
            fix: fixer => fixer.replaceTextRange([node.source.range[0] + 1, node.source.range[1] - 1], rewrittenImport),
            message: `Unexpected parent import '${sourcePath}'. Use '${rewrittenImport}' instead`,
            node
          });
        }

        const importWithoutAlias = resolvePath(sourcePath, currentFile, options);

        if (!(_importWithoutAlias = importWithoutAlias, isParentImport(_importWithoutAlias)) && hasAlias) {
          return context.report({
            fix: fixer => fixer.replaceTextRange([node.source.range[0] + 1, node.source.range[1] - 1], importWithoutAlias),
            message: `Unexpected subpath import via alias '${sourcePath}'. Use '${importWithoutAlias}' instead`,
            node
          });
        }

        return undefined;
      }
    };
  },
  meta: {
    fixable: true,
    schema: [{
      additionalProperties: false,
      properties: {
        alias: {
          type: 'object'
        }
      },
      type: 'object'
    }],
    type: 'suggestion'
  }
};
exports.default = _default;
module.exports = exports.default;